Data <- read.table("C:/Users/Phan Thu Trang/Downloads/Summer school 2018/Long distance.txt")
colnames(Data) <- c("Mode_choice", "Trip_duration", "Avail_R", "Access_R", "Avail_B", "Access_B")
## Count the number of data rows
hh <- nrow(Data)
## Set the initial values of the parameters (the number in parenthesis corresponds to the number of parameters to be estimated)
 b0 <- numeric(4)
## Recall that we estimate parameters by maximizing the log-likelihood function, hence we first need to define that function

##### Define the log-likelihood function of the logit model#####

fr <- function(x) {

  ## declare the log-likelihood variable, set value to 0
  LL = 0
  
  ### For this choice problem, we consider the following 3 modes
  ## rail
  ## bus
  ## car

  ## calculate the utility function: :introduce the desired explanatory variables in the function
                                       
  train <- Data$Avail_R * exp(x[1]*matrix(1,nrow = hh,ncol = 1)+ x[3] * Data$Access_R)
  bus <- Data$Avail_B * exp(x[2]*matrix(1,nrow = hh,ncol = 1)+ x[3] * Data$Access_B )
  car <- exp(x[3] * Data$Access_B + x[4] * Data$Trip_duration)
  ### Calculate the choice probabilities
  ## calculate the Inclusive Value (the denominator of the choice probabilities equation)
  deno <- (car + train + bus)
  
  ## Calculate indiviudal choice probabilities
  Ptrain <- Data$Avail_R * (train / deno)
  Pbus   <- Data$Avail_B * (bus / deno)
  Pcar   <- 1 * (car / deno)

  ## Avoid problems stemming from choice probabilities becoming zero.
  Ptrain <- (Ptrain !=0) * Ptrain + (Ptrain == 0)
  Pbus   <- (Pbus !=0) * Pbus     + (Pbus == 0)
  Pcar   <- (Pcar !=0) * Pcar   + (Pcar == 0)

  ## Choice results
  Ctrain   <- Data$Mode_choice == 200
  Cbus     <- Data$Mode_choice == 240
  Ccar     <- Data$Mode_choice == 100

  ## Calculate the Log-likelihood function
  LL <- colSums(Ctrain*log(Ptrain) + Cbus*log(Pbus) + Ccar  *log(Pcar))
  return(LL)
}
##### Maximize the Log-likelihood function#####

##Parameter optimization 
res <- optim(b0,fr,gr=NULL, method = "Nelder-Mead", hessian = TRUE, control=list(fnscale=-1))

## Parameters and Hessian matrix calculation 
b   <- res$par
hhh <- res$hessian

## Calculate the t-statistic
tval <- b/sqrt(-diag(solve(hhh)))

## L(0), Log-Likelihood when all parameters are 0
L0 <- fr(b0)
## LL, maximium likelihood
LL <- res$value

##### Output #####
print(res)
## t-statistic 
print(tval)
## L(0)
print(L0)
## LL
print(LL)
##rho-square
print((L0-LL)/L0)
## adjusted rho-square
print((L0-(LL-length(b)))/L0)
